/*
 * Include File Declarations
 */

#include <stdio.h>
#include <Xm/MainW.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/List.h>
#include <Xm/PushBG.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/MessageB.h>
#include <Xm/SelectioB.h>
#include <Xm/CascadeB.h>

/*-------------------------------------------------------------
** Forward Declarations
*/

void main();
Widget CreateHelp ();
void HelpCB ();
void QuitCB ();
void BrowseSelCB ();

/*
 * Global Variables
 */

static XmStringCharSet charset = (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;

Widget appshell, list1, error, info, prompt, query, warn, work; 
Arg args [15];
int n;

int i;

/*
 * These are the strings that are displayed by the list. They are
 * converted into compound strings at run time by the main program.
 */
static char  *CharListItems[7] = {
 "Create Error Dialog",
 "Create Information Dialog",
 "Create Prompt Dialog",
 "Create Question Dialog",
 "Create Warning Dialog",
 "Create Working Dialog",
 "Quit"
 };

#define NUM_LIST_ITEMS 7

XmString ListItems[NUM_LIST_ITEMS];

/*
 * This item is initially selected (highlighted) by the list.
 */
static char *CharSelectedItems[1] = {
 "Create Error Dialog"};

#define NUM_SELECTED_ITEMS 1
XmString SelectedItems[NUM_SELECTED_ITEMS];

XmStringCharSet cs = "ISOLatin1";

/*-------------------------------------------------------------
**  HelpCB      - callback for help button
*/
void HelpCB (w, client_data, call_data) 
Widget    w;    /*  widget id    */
XtPointer    client_data;  /*  data from application   */
XtPointer    call_data;  /*  data from widget class  */
{
  Widget    message_box;    /*  message box    */


  /*  Create help window. */
  message_box = CreateHelp (w);


  /*  Display help window. */
  XtManageChild (message_box);
}

/*-------------------------------------------------------------
**  CreateHelp    - create help window
*/
Widget CreateHelp (parent) 
  Widget    parent;    /*  parent widget  */
{
  Widget    button;
  Widget    message_box;  /*  Message Dialog   */
  Arg    args[10];    /*  arg list    */
  register int  n;    /*  arg count    */

  static char  message[1000];  /*  help text  */
  XmString  title_string = NULL;
  XmString  message_string = NULL;
  XmString  button_string = NULL;

  /*  Generate message to display. */
  sprintf (message, "\
This program shows you how to create and use a list widget.  When\n\
you select one of the items, the corresponding dialog box pops up with \n\
an appropriate message.  Press the 'OK' button at the bottom of the\n\
dialog box to remove that dialog.  To terminate the program, select\n\
the 'Quit' item or press the Exit button and then the 'Quit' button.\0");

  /* Create the compound strings */
  message_string = XmStringCreateLtoR (message, charset);
  button_string = XmStringCreateLtoR ("Close", charset);
  title_string = XmStringCreateLtoR ("getdlog help", charset);

  /*  Create message box dialog. */
  n = 0;
  XtSetArg (args[n], XmNdialogTitle, title_string);  n++;
  XtSetArg (args[n], XmNokLabelString, button_string);  n++;
  XtSetArg (args[n], XmNmessageString, message_string);  n++;
  message_box = XmCreateMessageDialog (parent, "helpbox", args, n);

  button = XmMessageBoxGetChild (message_box, XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild (button);
  button = XmMessageBoxGetChild (message_box, XmDIALOG_HELP_BUTTON);
  XtUnmanageChild (button);

  /*  Free strings and return message box. */
  if (title_string) XtFree (title_string);
  if (message_string) XtFree (message_string);
  if (button_string) XtFree (button_string);
  return (message_box);
}

/*-------------------------------------------------------------
**  QuitCB      - callback for quit button
*/
void QuitCB (w, client_data, call_data) 
Widget    w;    /*  widget id    */
XtPointer    client_data;  /*  data from application   */
XtPointer    call_data;  /*  data from widget class  */
{

  /*  Terminate the application. */
  exit (0);
}

/*
 * BrowseSelCB is the browse selection callback.  It functions as a
 * big case statement, comparing the item that was selected to
 * the items in the list.  When it finds a match, it takes the
 * appropriate action.
 */
void  BrowseSelCB(w,client_data,call_data)
  Widget w;
  XtPointer  client_data, call_data;
{
  XmListCallbackStruct *cb =  (XmListCallbackStruct *)call_data;
    
  if (XmStringCompare(cb->item,ListItems[0]))  /* Create Error Dialog */
  {
    n = 0;
    error = XmCreateMessageDialog (appshell, "error", args, n);
    XtManageChild (error);

    n = 0;
    XtSetArg (args[n], XmNx, 20);  n++;
    XtSetArg (args[n], XmNy, 40);  n++;
	XtSetValues (XtParent(error), args, n);
  }
    
  if (XmStringCompare(cb->item,ListItems[1]))  /* Create Information
  Dialog */
  {
    n = 0;
    info = XmCreateInformationDialog (appshell, "info", args, n);
    XtManageChild (info);

    n = 0;
    XtSetArg (args[n], XmNx, 20);  n++;
    XtSetArg (args[n], XmNy, 200);  n++;
	XtSetValues (XtParent(info), args, n);
  }

  if (XmStringCompare(cb->item,ListItems[2]))  /* Create Prompt Dialog */
  {                                 
    n = 0;
    prompt = XmCreatePromptDialog (appshell, "prompt", args, n);
    XtManageChild (prompt);

    n = 0;
    XtSetArg (args[n], XmNx, 20);  n++;
    XtSetArg (args[n], XmNy, 360);  n++;
	XtSetValues (XtParent(prompt), args, n);
  }

  if (XmStringCompare(cb->item,ListItems[3]))  /* Create Question Dialog */
  {
    n = 0;
    query = XmCreateQuestionDialog (appshell, "query", args, n);
    XtManageChild (query);

    n = 0;
    XtSetArg (args[n], XmNx, 20);  n++;
    XtSetArg (args[n], XmNy, 560);  n++;
	XtSetValues (XtParent(query), args, n);
  }

  if (XmStringCompare(cb->item,ListItems[4]))  /* Create a Warning Dialog */
  {
    n = 0;
    warn = XmCreateWarningDialog (appshell, "warn", args, n);
    XtManageChild (warn);

    n = 0;
    XtSetArg (args[n], XmNx, 20);  n++;
    XtSetArg (args[n], XmNy, 720);  n++;
	XtSetValues (XtParent(warn), args, n);
  }
    
  if (XmStringCompare(cb->item,ListItems[5])) /* Create a Working Dialog */
  {                                                 
    n = 0;
    work = XmCreateWorkingDialog (appshell, "work", args, n);
    XtManageChild (work);

    n = 0;
    XtSetArg (args[n], XmNx, 20);  n++;
    XtSetArg (args[n], XmNy, 890);  n++;
	XtSetValues (XtParent(work), args, n);
  }

  if (XmStringCompare(cb->item,ListItems[6]))  /* End the program. */
  {
    exit(0);
  }

}

/*
 *MAIN  
 */
void main(argc, argv)
 unsigned int argc;
 char **argv;
{
  Widget  main_window, form, frame, menu_bar, menu_pane, cascade, button;
  XmString label_text1, label_text2, label_text3;
  XtAppContext app_context;
  Display *display;
   
/*
 * Initialize the application shell widget
 */
  appshell = XtAppInitialize(&app_context, "Getdlog", NULL, 0, &argc, argv,
  NULL, args, 0);

 /* Create main window.
 */
 n = 0;
 main_window = XmCreateMainWindow (appshell, "main1", args, n);
 XtManageChild (main_window);

 /* Create menu bar in main window. */
 n = 0;
 menu_bar = XmCreateMenuBar (main_window, "menu_bar", args, n); 
 XtManageChild (menu_bar);

 /* Create Exit pulldown menu. */
 n = 0;
 menu_pane = XmCreatePulldownMenu (menu_bar, "menu_pane", args, n);

 n = 0;
 button = XmCreatePushButtonGadget (menu_pane, "Quit", args, n);
 XtManageChild (button);
 XtAddCallback (button, XmNactivateCallback, QuitCB, NULL);

 n = 0;
 XtSetArg (args[n], XmNsubMenuId, menu_pane); n++;
 cascade = XmCreateCascadeButton (menu_bar, "Exit", args, n);
 XtManageChild (cascade);

 /* Create "Help" button. */
 n = 0;
 cascade = XmCreateCascadeButton (menu_bar, "Help", args, n);
 XtManageChild (cascade);
 XtAddCallback (cascade, XmNactivateCallback, HelpCB, NULL);

 n = 0;
 XtSetArg (args[n], XmNmenuHelpWidget, cascade); n++;
 XtSetValues (menu_bar, args, n);

 i = 0; 
    
/* Create a form widget */
 i = 0;
 form = XmCreateForm (main_window, "form", args, i);

/* Manage the form widget */
  XtManageChild(form);

/*
 * Set main window areas 
 */
  XmMainWindowSetAreas (main_window, menu_bar, NULL, NULL, NULL, form);

/* Create a frame widget to enhance the list's appearance */
  i = 0;
  XtSetArg (args[i], XmNbottomOffset, 15); i++;
  XtSetArg (args[i], XmNbottomAttachment, XmATTACH_FORM); i++;
  XtSetArg (args[i], XmNtopOffset, 15); i++;
  XtSetArg (args[i], XmNtopAttachment, XmATTACH_FORM); i++;
  XtSetArg (args[i], XmNleftOffset, 15); i++;
  XtSetArg (args[i], XmNleftAttachment, XmATTACH_FORM); i++;
  XtSetArg (args[i], XmNrightOffset, 15); i++;
  XtSetArg (args[i], XmNrightAttachment, XmATTACH_FORM); i++;
  XtSetArg (args[i], XmNmarginHeight, 10); i++;
  XtSetArg (args[i], XmNmarginWidth, 10); i++;
  frame = XmCreateFrame (form, "frame", args, i);

/* Manage the frame widget */
  XtManageChild(frame);

/*
 * Create the arrays of compound strings from the character arrays.
 */
 for(i = 0; i < NUM_LIST_ITEMS; i++)
   ListItems[i] = (XmString )XmStringCreateLtoR(CharListItems[i],cs);
 for(i = 0; i < NUM_SELECTED_ITEMS; i++)
   SelectedItems[i] = (XmString )XmStringCreateLtoR(CharSelectedItems[i],cs);

/*
 * Create a list widget
 */
  i = 0;
  XtSetArg (args[i], XmNmarginHeight, 10); i++;
  XtSetArg (args[i], XmNmarginWidth, 10); i++;
  XtSetArg (args[i], XmNselectedItems, SelectedItems); i++;
  XtSetArg (args[i], XmNselectedItemCount, NUM_SELECTED_ITEMS); i++;
  XtSetArg (args[i], XmNvisibleItemCount, NUM_LIST_ITEMS); i++;    
  XtSetArg (args[i], XmNitems, ListItems); i++;    
  XtSetArg (args[i], XmNitemCount, NUM_LIST_ITEMS); i++; 

  list1 = XmCreateList(frame, "list1", args, i);

  XtManageChild(list1);

/*
 * Add the callback for browse selection.
 */
  XtAddCallback(list1, XmNbrowseSelectionCallback, BrowseSelCB, NULL);

/*
 * Realize the appshell widget, which displays all children
 */ 
  XtRealizeWidget(appshell);

/*
 * Go into a loop and wait for input
 */
  XtAppMainLoop(app_context);
}
